@@ -16,7 +16,8 @@ Models that vertically concatenate two or more table models.
 :date:   9.10.2019
 """
 
-from PySide2.QtCore import Qt, Signal, Slot, QModelIndex
+import bisect
+from PySide2.QtCore import Qt, Signal, Slot, QModelIndex, QTimer
 from ..mvcmodels.minimal_table_model import MinimalTableModel
 
 
@@ -29,7 +30,8 @@ class CompoundTableModel(MinimalTableModel):
         """Initializes model.
 
         Args:
-            parent (QObject): the parent object
+            parent (QObject, optional): the parent object
+            header (list of str, optional): header labels
         """
         super().__init__(parent=parent, header=header)
         self.sub_models = []
@@ -122,6 +124,19 @@ class CompoundTableModel(MinimalTableModel):
             self._inv_row_map[model_row_tup] = self.rowCount()
             self._row_map.append(model_row_tup)
 
+    def _row_map_iterator_for_model(self, model):
+        """Yields row map for given model.
+        The base class implementation just yields all model rows.
+
+        Args:
+            model (MinimalTableModel)
+
+        Yields:
+            tuple: (model, row number)
+        """
+        for i in range(model.rowCount()):
+            yield (model, i)
+
     def _row_map_for_model(self, model):
         """Returns row map for given model.
         The base class implementation just returns all model rows.
@@ -132,16 +147,16 @@ class CompoundTableModel(MinimalTableModel):
         Returns:
             list: tuples (model, row number)
         """
-        return [(model, i) for i in range(model.rowCount())]
+        return list(self._row_map_iterator_for_model(model))
 
-    def canFetchMore(self, parent=QModelIndex()):
+    def canFetchMore(self, parent):
         """Returns True if any of the submodels that haven't been fetched yet can fetch more."""
         for self._next_sub_model in self.sub_models:
             if self._next_sub_model.canFetchMore(self.map_to_sub(parent)):
                 return True
         return False
 
-    def fetchMore(self, parent=QModelIndex()):
+    def fetchMore(self, parent):
         """Fetches the next sub model and increments the fetched counter."""
         self._next_sub_model.fetchMore(self.map_to_sub(parent))
         if not self._next_sub_model.rowCount():
@@ -194,10 +209,11 @@ class CompoundTableModel(MinimalTableModel):
             return False
         if count < 1:
             return False
-        try:
+        if row < self.rowCount():
             sub_model, sub_row = self._row_map[row]
-        except IndexError:
+        else:
             sub_model, sub_row = self._row_map[-1]
+            sub_row += 1
         self.beginInsertRows(parent, row, row + count - 1)
         sub_model.insertRows(sub_row, count, self.map_to_sub(parent))
         self.endInsertRows()
@@ -243,35 +259,33 @@ class CompoundWithEmptyTableModel(CompoundTableModel):
     def empty_model(self):
         return self.sub_models[-1]
 
-    def _create_single_models(self):
-        """Returns a list of single models."""
-        raise NotImplementedError()
-
     def _create_empty_model(self):
-        """Returns an empty model."""
+        """Creates and returns an empty model.
+
+        Returns:
+            EmptyRowModel: model
+        """
         raise NotImplementedError()
 
     def init_model(self):
-        """Initializes the compound model. Basically populates the sub_models list attribute
-        with the result of _create_single_models and _create_empty_model.
+        """Initializes the compound model.
+
+        Basically populates the sub_models list attribute with the result of _create_empty_model.
         """
         self.clear_model()
-        self.sub_models = self._create_single_models()
         self.sub_models.append(self._create_empty_model())
-        self.connect_model_signals()
-
-    def connect_model_signals(self):
-        """Connects signals so changes in the submodels are acknowledge by the compound."""
         self.empty_model.rowsRemoved.connect(self._handle_empty_rows_removed)
         self.empty_model.rowsInserted.connect(self._handle_empty_rows_inserted)
-        for model in self.single_models:
-            model.modelReset.connect(lambda model=model: self._handle_single_model_reset(model))
-        for model in self.sub_models:
-            model.dataChanged.connect(
-                lambda top_left, bottom_right, roles, model=model: self.dataChanged.emit(
-                    self.map_from_sub(model, top_left), self.map_from_sub(model, bottom_right), roles
-                )
+
+    def _connect_single_model(self, model):
+        """Connects signals so changes in the submodels are acknowledged by the compound."""
+        model.modelReset.connect(lambda model=model: self._handle_single_model_reset(model))
+        model.modelAboutToBeReset.connect(lambda model=model: self._handle_single_model_about_to_be_reset(model))
+        model.dataChanged.connect(
+            lambda top_left, bottom_right, roles, model=model: self.dataChanged.emit(
+                self.map_from_sub(model, top_left), self.map_from_sub(model, bottom_right), roles
             )
+        )
 
     def _recompute_empty_row_map(self):
         """Recomputes the part of the row map corresponding to the empty model."""
@@ -283,7 +297,7 @@ class CompoundWithEmptyTableModel(CompoundTableModel):
             pass
         self._append_row_map(empty_row_map)
 
-    @Slot("QModelIndex", "int", "int")
+    @Slot(QModelIndex, int, int)
     def _handle_empty_rows_removed(self, parent, empty_first, empty_last):
         """Runs when rows are removed from the empty model.
         Updates row_map, then emits rowsRemoved so the removed rows are no longer visible.
@@ -293,7 +307,7 @@ class CompoundWithEmptyTableModel(CompoundTableModel):
         self._recompute_empty_row_map()
         self.rowsRemoved.emit(QModelIndex(), first, last)
 
-    @Slot("QModelIndex", "int", "int")
+    @Slot(QModelIndex, int, int)
     def _handle_empty_rows_inserted(self, parent, empty_first, empty_last):
         """Runs when rows are inserted to the empty model.
         Updates row_map, then emits rowsInserted so the new rows become visible.
@@ -303,25 +317,61 @@ class CompoundWithEmptyTableModel(CompoundTableModel):
         last = self._inv_row_map[self.empty_model, empty_last]
         self.rowsInserted.emit(QModelIndex(), first, last)
 
-    def _handle_single_model_reset(self, single_model):
-        """Runs when one of the single models is reset.
-        Updates row_map, then emits rowsInserted so the new rows become visible.
-        """
-        single_row_map = self._row_map_for_model(single_model)
-        self._insert_single_row_map(single_row_map)
-
-    def _insert_single_row_map(self, single_row_map):
-        """Inserts given row map just before the empty model's."""
-        if not single_row_map:
+    def _handle_single_model_about_to_be_reset(self, model):
+        """Runs when given model is about to reset."""
+        if model not in self.single_models:
             return
-        try:
-            row = self._inv_row_map[self.empty_model, 0]
-            self._row_map, empty_row_map = self._row_map[:row], self._row_map[row:]
-        except KeyError:
-            row = self.rowCount()
-            empty_row_map = []
+        row_map = self._row_map_for_model(model)
+        if not row_map:
+            return
+        first = self._inv_row_map[row_map[0]]
+        last = first + len(row_map) - 1
+        tail_row_map = self._row_map[last + 1 :]
+        for key in self._row_map[first:]:
+            self._inv_row_map.pop(key)
+        self._row_map[first:] = []
+        self._append_row_map(tail_row_map)
+        self.rowsRemoved.emit(QModelIndex(), first, last)
+
+    def _handle_single_model_reset(self, model):
+        """Runs when given model is reset."""
+        if model in self.single_models:
+            self._refresh_single_model(model)
+        else:
+            self._insert_single_model(model)
+
+    def _refresh_single_model(self, model):
+        single_row_map = self._row_map_for_model(model)
+        pos = self.single_models.index(model) + 1
+        self._insert_row_map(pos, single_row_map)
+
+    def _get_insert_position(self, model):
+        return bisect.bisect_left(self.single_models, model)
+
+    def _insert_single_model(self, model):
+        single_row_map = self._row_map_for_model(model)
+        pos = self._get_insert_position(model)
+        self._insert_row_map(pos, single_row_map)
+        self.sub_models.insert(pos, model)
+
+    def _get_row_for_insertion(self, pos):
+        for model in self.sub_models[pos:]:
+            first_row_map_item = next(self._row_map_iterator_for_model(model), None)
+            if first_row_map_item is not None:
+                return self._inv_row_map[first_row_map_item]
+        return self.rowCount()
+
+    def _insert_row_map(self, pos, single_row_map):
+        if not single_row_map:
+            # To trigger fetching. The QTimer is to avoid funny situations where the user enters new data
+            # via the empty row model, and those rows need to be removed at the same time as we fetch the added data.
+            # Doing it in the same loop cycle causes bugs.
+            QTimer.singleShot(0, self.layoutChanged.emit)
+            return
+        row = self._get_row_for_insertion(pos)
+        self._row_map, tail_row_map = self._row_map[:row], self._row_map[row:]
         self._append_row_map(single_row_map)
-        self._append_row_map(empty_row_map)
+        self._append_row_map(tail_row_map)
         first = row
         last = row + len(single_row_map) - 1
         self.rowsInserted.emit(QModelIndex(), first, last)
