@@ -16,7 +16,7 @@ Contains SpecificationEditorWindowBase and ChangeSpecPropertyCommand
 :date:   12.4.2018
 """
 
-from PySide2.QtGui import QGuiApplication, QKeySequence, QIcon
+from PySide2.QtGui import QKeySequence, QIcon
 from PySide2.QtCore import Slot, Qt
 from PySide2.QtWidgets import (
     QMainWindow,
@@ -33,9 +33,8 @@ from PySide2.QtWidgets import (
     QUndoCommand,
     QErrorMessage,
 )
-from spinetoolbox.config import STATUSBAR_SS
-from spinetoolbox.widgets.notification import ChangeNotifier
-from spinetoolbox.helpers import ensure_window_is_on_screen, CharIconEngine
+from spinetoolbox.widgets.notification import ChangeNotifier, Notification
+from spinetoolbox.helpers import CharIconEngine, restore_ui, save_ui
 
 
 class ChangeSpecPropertyCommand(QUndoCommand):
@@ -84,6 +83,7 @@ class SpecificationEditorWindowBase(QMainWindow):
         self._ui = self._make_ui()
         self._ui.setupUi(self)
         self._ui_error = QErrorMessage(self)
+        self._ui_error.setWindowFlag(Qt.WindowContextHelpButtonHint, False)
         self._ui_error.setWindowTitle("Error")
         self.setWindowTitle(specification.name if specification else "")
         # Restore ui
@@ -94,8 +94,8 @@ class SpecificationEditorWindowBase(QMainWindow):
         self._change_notifier = ChangeNotifier(self, self._undo_stack, self._app_settings, "appSettings/specShowUndo")
         # Setup toolbar
         self._spec_toolbar = _SpecNameDescriptionToolbar(self, specification, self._undo_stack)
+        self._spec_toolbar.show_toolbox_action.triggered.connect(self._toolbox.restore_and_activate)
         self.addToolBar(Qt.TopToolBarArea, self._spec_toolbar)
-        self._ui.statusbar.setStyleSheet(STATUSBAR_SS)
         self._populate_main_menu()
         self._spec_toolbar.save_action.triggered.connect(self._save)
         self._spec_toolbar.duplicate_action.triggered.connect(self._duplicate)
@@ -135,14 +135,14 @@ class SpecificationEditorWindowBase(QMainWindow):
         raise NotImplementedError()
 
     @Slot(str)
-    def _show_error(self, message):
+    def show_error(self, message):
         self._ui_error.showMessage(message)
 
     def _show_status_bar_msg(self, msg):
         word_count = len(msg.split(" "))
         mspw = 60000 / 140  # Assume we can read ~140 words per minute
         duration = mspw * word_count
-        self._ui.statusbar.showMessage(msg, duration)
+        Notification(self, msg, life_span=duration, corner=Qt.BottomRightCorner).show()
 
     def _populate_main_menu(self):
         undo_action = self._undo_stack.createUndoAction(self)
@@ -166,18 +166,18 @@ class SpecificationEditorWindowBase(QMainWindow):
             bool: True if operation was successful, False otherwise
         """
         if not self._toolbox.project():
-            self._show_error("Please open or create a project first")
+            self.show_error("Please open or create a project first")
             return False
         name = self._spec_toolbar.name()
         if not name:
-            self._show_error("Please enter a name for the specification.")
+            self.show_error("Please enter a name for the specification.")
             return False
         spec = self._make_new_specification(name)
         if spec is None:
             return False
         if not self._original_spec_name:
             if self._toolbox.project().is_specification_name_reserved(name):
-                self._show_error("Specification name already in use. Please enter a new name.")
+                self.show_error("Specification name already in use. Please enter a new name.")
                 return False
             self._toolbox.add_specification(spec)
             if not self._toolbox.project().is_specification_name_reserved(name):
@@ -186,7 +186,7 @@ class SpecificationEditorWindowBase(QMainWindow):
                 self.item.set_specification(spec)
         else:
             if name != self._original_spec_name and self._toolbox.project().is_specification_name_reserved(name):
-                self._show_error("Specification name already in use. Please enter a new name.")
+                self.show_error("Specification name already in use. Please enter a new name.")
                 return False
             spec.definition_file_path = self.specification.definition_file_path
             self._toolbox.replace_specification(self._original_spec_name, spec)
@@ -205,7 +205,7 @@ class SpecificationEditorWindowBase(QMainWindow):
 
     def _duplicate(self):
         if not self._toolbox.project():
-            self._show_error("Please open or create a project first")
+            self.show_error("Please open or create a project first")
             return
         new_spec = self._make_new_specification("")
         self._toolbox.show_specification_form(new_spec.item_type, new_spec, **self._duplicate_kwargs)
@@ -259,6 +259,9 @@ class _SpecNameDescriptionToolbar(QToolBar):
         layout.setStretchFactor(self._line_edit_name, 1)
         layout.setStretchFactor(self._line_edit_description, 3)
         self.addWidget(widget)
+        toolbox_icon = QIcon(":/symbols/Spine_symbol.png")
+        self.show_toolbox_action = self.addAction(toolbox_icon, "Show Spine Toolbox (Ctrl+ESC)")
+        self.show_toolbox_action.setShortcut(Qt.CTRL + Qt.Key_Escape)
         self.menu = self._make_main_menu()
         self.save_action = self.menu.addAction("Save")
         self.duplicate_action = self.menu.addAction("Duplicate")
@@ -361,51 +364,3 @@ def prompt_to_save_changes(parent, settings, save_callback):
     if answer == QMessageBox.Yes:
         return save_callback()
     return True
-
-
-def restore_ui(window, app_settings, settings_group):
-    """Restores UI state from previous session.
-
-    Args:
-        window (QMainWindow)
-        app_settings (QSettings)
-        settings_group (str)
-    """
-    app_settings.beginGroup(settings_group)
-    window_size = app_settings.value("windowSize")
-    window_pos = app_settings.value("windowPosition")
-    window_state = app_settings.value("windowState")
-    window_maximized = app_settings.value("windowMaximized", defaultValue='false')
-    n_screens = app_settings.value("n_screens", defaultValue=1)
-    app_settings.endGroup()
-    original_size = window.size()
-    if window_size:
-        window.resize(window_size)
-    if window_pos:
-        window.move(window_pos)
-    if window_state:
-        window.restoreState(window_state, version=1)  # Toolbar and dockWidget positions
-    # noinspection PyArgumentList
-    if len(QGuiApplication.screens()) < int(n_screens):
-        # There are less screens available now than on previous application startup
-        window.move(0, 0)  # Move this widget to primary screen position (0,0)
-    ensure_window_is_on_screen(window, original_size)
-    if window_maximized == 'true':
-        window.setWindowState(Qt.WindowMaximized)
-
-
-def save_ui(window, app_settings, settings_group):
-    """Saves UI state for next session.
-
-    Args:
-        window (QMainWindow)
-        app_settings (QSettings)
-        settings_group (str)
-    """
-    app_settings.beginGroup(settings_group)
-    app_settings.setValue("windowSize", window.size())
-    app_settings.setValue("windowPosition", window.pos())
-    app_settings.setValue("windowState", window.saveState(version=1))
-    app_settings.setValue("windowMaximized", window.windowState() == Qt.WindowMaximized)
-    app_settings.setValue("n_screens", len(QGuiApplication.screens()))
-    app_settings.endGroup()
