@@ -17,18 +17,8 @@ QUndoCommand subclasses for modifying the db.
 """
 
 import time
-from PySide2.QtCore import Slot
 from PySide2.QtWidgets import QUndoCommand, QUndoStack
-from spinetoolbox.helpers import signal_waiter
-
-
-def _format_item(item_type, item):
-    return {
-        "parameter_value": lambda x: "<"
-        + ", ".join([x.get("object_name") or x.get("object_name_list"), x["parameter_name"]])
-        + ">",
-        "parameter_definition": lambda x: x.get("parameter_name") or x.get("name"),
-    }.get(item_type, lambda x: x["name"])(item)
+from spinetoolbox.helpers import signal_waiter, separate_metadata_and_item_metadata
 
 
 class AgedUndoStack(QUndoStack):
@@ -80,12 +70,16 @@ class SpineDBCommand(AgedUndoCommand):
         "parameter_definition": "add parameter definitions",
         "parameter_value": "add parameter values",
         "parameter_value_list": "add parameter value lists",
+        "list_value": "add parameter value list values",
         "alternative": "add alternative",
         "scenario": "add scenario",
         "feature": "add feature",
         "tool": "add tool",
         "tool_feature": "add tool features",
         "tool_feature_method": "add tool feature methods",
+        "metadata": "add metadata",
+        "entity_metadata": "add entity metadata",
+        "parameter_value_metadata": "add parameter value metadata",
     }
     _update_command_name = {
         "object_class": "update object classes",
@@ -95,12 +89,16 @@ class SpineDBCommand(AgedUndoCommand):
         "parameter_definition": "update parameter definitions",
         "parameter_value": "update parameter values",
         "parameter_value_list": "update parameter value lists",
-        "alternative": "update alternative",
-        "scenario": "update scenario",
-        "feature": "update feature",
-        "tool": "update tool",
+        "list_value": "update parameter value list values",
+        "alternative": "update alternatives",
+        "scenario": "update scenarios",
+        "feature": "update features",
+        "tool": "update tools",
         "tool_feature": "update tool features",
         "tool_feature_method": "update tool feature methods",
+        "metadata": "update metadata",
+        "entity_metadata": "update entity metadata",
+        "parameter_value_metadata": "update parameter value metadata",
     }
     _add_method_name = {
         "object_class": "add_object_classes",
@@ -110,7 +108,8 @@ class SpineDBCommand(AgedUndoCommand):
         "entity_group": "add_entity_groups",
         "parameter_definition": "add_parameter_definitions",
         "parameter_value": "add_parameter_values",
-        "parameter_value_list": "add_wide_parameter_value_lists",
+        "parameter_value_list": "add_parameter_value_lists",
+        "list_value": "add_list_values",
         "alternative": "add_alternatives",
         "scenario": "add_scenarios",
         "scenario_alternative": "add_scenario_alternatives",
@@ -118,6 +117,9 @@ class SpineDBCommand(AgedUndoCommand):
         "tool": "add_tools",
         "tool_feature": "add_tool_features",
         "tool_feature_method": "add_tool_feature_methods",
+        "metadata": "add_metadata",
+        "entity_metadata": "add_ext_entity_metadata",
+        "parameter_value_metadata": "add_ext_parameter_value_metadata",
     }
     _update_method_name = {
         "object_class": "update_object_classes",
@@ -126,14 +128,18 @@ class SpineDBCommand(AgedUndoCommand):
         "relationship": "update_wide_relationships",
         "parameter_definition": "update_parameter_definitions",
         "parameter_value": "update_parameter_values",
-        "parameter_value_list": "update_wide_parameter_value_lists",
+        "parameter_value_list": "update_parameter_value_lists",
+        "list_value": "update_list_values",
         "alternative": "update_alternatives",
         "scenario": "update_scenarios",
-        "scenario_alternative": "_update_scenario_alternatives",
+        "scenario_alternative": "update_scenario_alternatives",
         "feature": "update_features",
         "tool": "update_tools",
         "tool_feature": "update_tool_features",
         "tool_feature_method": "update_tool_feature_methods",
+        "metadata": "update_metadata",
+        "entity_metadata": "update_ext_entity_metadata",
+        "parameter_value_metadata": "update_ext_parameter_value_metadata",
     }
     _added_signal_name = {
         "object_class": "object_classes_added",
@@ -144,6 +150,7 @@ class SpineDBCommand(AgedUndoCommand):
         "parameter_definition": "parameter_definitions_added",
         "parameter_value": "parameter_values_added",
         "parameter_value_list": "parameter_value_lists_added",
+        "list_value": "list_values_added",
         "alternative": "alternatives_added",
         "scenario": "scenarios_added",
         "scenario_alternative": "scenario_alternatives_added",
@@ -151,6 +158,9 @@ class SpineDBCommand(AgedUndoCommand):
         "tool": "tools_added",
         "tool_feature": "tool_features_added",
         "tool_feature_method": "tool_feature_methods_added",
+        "metadata": "metadata_added",
+        "entity_metadata": "entity_metadata_added",
+        "parameter_value_metadata": "parameter_value_metadata_added",
     }
     _updated_signal_name = {
         "object_class": "object_classes_updated",
@@ -160,6 +170,7 @@ class SpineDBCommand(AgedUndoCommand):
         "parameter_definition": "parameter_definitions_updated",
         "parameter_value": "parameter_values_updated",
         "parameter_value_list": "parameter_value_lists_updated",
+        "list_value": "list_values_updated",
         "alternative": "alternatives_updated",
         "scenario": "scenarios_updated",
         "scenario_alternative": "scenario_alternatives_updated",
@@ -167,6 +178,9 @@ class SpineDBCommand(AgedUndoCommand):
         "tool": "tools_updated",
         "tool_feature": "tool_features_updated",
         "tool_feature_method": "tool_feature_methods_updated",
+        "metadata": "metadata_updated",
+        "entity_metadata": "entity_metadata_updated",
+        "parameter_value_metadata": "parameter_value_metadata_updated",
     }
 
     def __init__(self, db_mngr, db_map, parent=None):
@@ -204,8 +218,7 @@ class SpineDBCommand(AgedUndoCommand):
 
     @staticmethod
     def undomethod(func):
-        """Returns a new undo method that silences the affected Spine db editors.
-        """
+        """Returns a new undo method that silences the affected Spine db editors."""
 
         def undo(self):
             super().undo()
@@ -216,10 +229,6 @@ class SpineDBCommand(AgedUndoCommand):
     def receive_items_changed(self, _):
         raise NotImplementedError()
 
-    def data(self):
-        """Returns data to present this command in a DBHistoryDialog."""
-        raise NotImplementedError()
-
 
 class AddItemsCommand(SpineDBCommand):
     def __init__(self, db_mngr, db_map, data, item_type, parent=None, check=True):
@@ -261,9 +270,13 @@ class AddItemsCommand(SpineDBCommand):
         self._readd = True
 
     def receive_items_changed(self, db_map_data):
+        if self.db_map not in db_map_data:
+            self.setObsolete(True)
+            return
+        if self.item_type == "entity_metadata" or self.item_type == "parameter_value_metadata":
+            db_map_data, _ = separate_metadata_and_item_metadata(db_map_data)
         self.redo_db_map_data = {
-            db_map: [self.db_mngr.cache_to_db(self.item_type, item) for item in data]
-            for db_map, data in db_map_data.items()
+            db_map: [db_map.cache_to_db(self.item_type, item) for item in data] for db_map, data in db_map_data.items()
         }
         self.undo_db_map_typed_ids = {
             db_map: db_map.cascading_ids(
@@ -273,9 +286,6 @@ class AddItemsCommand(SpineDBCommand):
             for db_map, data in db_map_data.items()
         }
 
-    def data(self):
-        return {_format_item(self.item_type, item): [] for item in self.redo_db_map_data[self.db_map]}
-
 
 class UpdateItemsCommand(SpineDBCommand):
     def __init__(self, db_mngr, db_map, data, item_type, parent=None, check=True):
@@ -305,7 +315,7 @@ class UpdateItemsCommand(SpineDBCommand):
 
     def _undo_item(self, db_map, id_):
         undo_item = self.db_mngr.get_item(db_map, self.item_type, id_)
-        return self.db_mngr.cache_to_db(self.item_type, undo_item)
+        return db_map.cache_to_db(self.item_type, undo_item)
 
     @SpineDBCommand.redomethod
     def redo(self):
@@ -320,15 +330,14 @@ class UpdateItemsCommand(SpineDBCommand):
         )
 
     def receive_items_changed(self, db_map_data):
+        if not db_map_data.get(self.db_map):
+            self.setObsolete(True)
+            return
         self.redo_db_map_data = {
-            db_map: [self.db_mngr.cache_to_db(self.item_type, item) for item in data]
-            for db_map, data in db_map_data.items()
+            db_map: [db_map.cache_to_db(self.item_type, item) for item in data] for db_map, data in db_map_data.items()
         }
         self._check = False
 
-    def data(self):
-        return {_format_item(self.item_type, item): [] for item in self.undo_db_map_data[self.db_map]}
-
 
 class RemoveItemsCommand(SpineDBCommand):
     def __init__(self, db_mngr, db_map, typed_data, parent=None):
@@ -365,15 +374,12 @@ class RemoveItemsCommand(SpineDBCommand):
                 waiter.wait()
 
     def receive_items_changed(self, typed_db_map_data):
+        if not any(db_map_data.get(self.db_map) for db_map_data in typed_db_map_data.values()):
+            self.setObsolete(True)
+            return
         self.undo_typed_db_map_data = {
             item_type: {
-                self.db_map: [self.db_mngr.cache_to_db(item_type, item) for item in db_map_data.get(self.db_map, [])]
+                self.db_map: [self.db_map.cache_to_db(item_type, item) for item in db_map_data.get(self.db_map, [])]
             }
             for item_type, db_map_data in typed_db_map_data.items()
         }
-
-    def data(self):
-        return {
-            item_type: [_format_item(item_type, item) for item in self.undo_typed_db_map_data[item_type][self.db_map]]
-            for item_type in reversed(list(self.undo_typed_db_map_data.keys()))
-        }
