@@ -91,8 +91,8 @@ class ObjectTreeModel(MultiDBTreeModel):
                         result.setdefault(parent_item, {}).setdefault(db_map, []).extend(list(ids.keys()))
         return result
 
-    def _parent_entity_group_data(self, db_map_data):
-        """Takes given entity group data and returns the same data keyed by parent tree-item.
+    def _parent_relationship_data_for_update(self, db_map_data):
+        """Takes given relationship data and returns the same data keyed by parent tree-item.
 
         Args:
             db_map_data (dict): maps DiffDatabaseMapping instances to list of items as dict
@@ -104,9 +104,9 @@ class ObjectTreeModel(MultiDBTreeModel):
         for db_map, items in db_map_data.items():
             d = dict()
             for item in items:
-                d.setdefault(item["class_id"], dict())[item["group_id"]] = None
+                d.setdefault(item["class_id"], dict())[item["id"]] = None
             for class_id, ids in d.items():
-                for parent_item in self.find_items(db_map, (class_id,)):
+                for parent_item in self.find_items(db_map, (None, None, class_id)):
                     result.setdefault(parent_item, {})[db_map] = list(ids.keys())
         return result
 
@@ -148,8 +148,6 @@ class ObjectTreeModel(MultiDBTreeModel):
     def add_entity_groups(self, db_map_data):
         for parent_item, db_map_ids in self._parent_entity_member_data(db_map_data).items():
             parent_item.append_children_by_id(db_map_ids)
-        for parent_item, db_map_ids in self._parent_entity_group_data(db_map_data).items():
-            parent_item.raise_group_children_by_id(db_map_ids)
 
     def remove_object_classes(self, db_map_data):
         db_map_ids = {db_map: {x["id"] for x in data} for db_map, data in db_map_data.items()}
@@ -184,7 +182,7 @@ class ObjectTreeModel(MultiDBTreeModel):
             parent_item.update_children_by_id(db_map_ids)
 
     def update_relationships(self, db_map_data):
-        for parent_item, db_map_ids in self._parent_relationship_data(db_map_data).items():
+        for parent_item, db_map_ids in self._parent_relationship_data_for_update(db_map_data).items():
             parent_item.update_children_by_id(db_map_ids)
 
     def find_next_relationship_index(self, index):
