@@ -23,7 +23,7 @@ from PySide2.QtGui import QCursor, QPainter, QIcon
 from PySide2.QtPrintSupport import QPrinter
 from ...helpers import CharIconEngine
 from ...widgets.custom_qgraphicsviews import CustomQGraphicsView
-from ...widgets.custom_qwidgets import ToolBarWidgetAction
+from ...widgets.custom_qwidgets import ToolBarWidgetAction, HorizontalSpinBox
 from ..graphics_items import EntityItem, ObjectItem, RelationshipItem, CrossHairsArcItem, make_figure_graphics_item
 from .select_position_parameters_dialog import SelectPositionParametersDialog
 from .graph_layout_generator import make_heat_map
@@ -46,17 +46,25 @@ class EntityQGraphicsView(CustomQGraphicsView):
         self.pos_x_parameter = "x"
         self.pos_y_parameter = "y"
         self.selected_items = list()
-        self.removed_items = list()
+        self.removed_items = set()
         self.hidden_items = dict()
-        self.pruned_entity_ids = dict()
+        self.pruned_db_map_entity_ids = dict()
         self.heat_map_items = list()
         self._point_value_tuples_per_parameter_name = dict()  # Used in the heat map menu
         self._hovered_obj_item = None
         self.relationship_class = None
         self.cross_hairs_items = []
         self.auto_expand_objects = None
+        self.merge_dbs = None
+        self.max_relationship_dimension = None
+        self.disable_max_relationship_dimension = None
         self._auto_expand_objects_action = None
+        self._merge_dbs_action = None
+        self._max_rel_dim_action = None
+        self._disable_max_rel_dim_action = None
+        self._max_rel_dim_spin_box = None
         self._add_objects_action = None
+        self._select_pos_param_action = None
         self._save_pos_action = None
         self._clear_pos_action = None
         self._hide_selected_action = None
@@ -75,7 +83,7 @@ class EntityQGraphicsView(CustomQGraphicsView):
         self._prune_classes_menu = None
         self._restore_pruned_menu = None
         self._parameter_heat_map_menu = None
-        self._items_per_db_map_class = {}
+        self._items_per_class = {}
 
     @property
     def _qsettings(self):
@@ -89,12 +97,8 @@ class EntityQGraphicsView(CustomQGraphicsView):
     def entity_items(self):
         return [x for x in self.scene().items() if isinstance(x, EntityItem) and x not in self.removed_items]
 
-    def setScene(self, scene):
-        super().setScene(scene)
-        scene.selectionChanged.connect(self._handle_scene_selection_changed)
-
     @Slot()
-    def _handle_scene_selection_changed(self):
+    def handle_scene_selection_changed(self):
         """Filters parameters by selected objects in the graph."""
         if self.scene() is None:
             return
@@ -103,22 +107,50 @@ class EntityQGraphicsView(CustomQGraphicsView):
         selected_rels = [x for x in selected_items if isinstance(x, RelationshipItem)]
         self.selected_items = selected_objs + selected_rels
         self.graph_selection_changed.emit({"object": selected_objs, "relationship": selected_rels})
+        default_data = selected_items[0].default_parameter_data() if len(selected_items) == 1 else {}
+        self._spine_db_editor.set_default_parameter_data(default_data)
 
     def connect_spine_db_editor(self, spine_db_editor):
         self._spine_db_editor = spine_db_editor
         self.populate_context_menu()
 
     def populate_context_menu(self):
+        self.auto_expand_objects = self._qsettings.value("appSettings/autoExpandObjects", defaultValue="true") == "true"
+        self.merge_dbs = self._qsettings.value("appSettings/mergeDBs", defaultValue="true") == "true"
+        self.max_relationship_dimension = int(
+            self._qsettings.value("appSettings/maxRelationshipDimension", defaultValue="2")
+        )
+        self.disable_max_relationship_dimension = (
+            self._qsettings.value("appSettings/disableMaxRelationshipDimension", defaultValue="true") == "true"
+        )
         self._auto_expand_objects_action = self._menu.addAction("Auto-expand objects")
         self._auto_expand_objects_action.setCheckable(True)
-        self.auto_expand_objects = (
-            self._spine_db_editor.qsettings.value("appSettings/autoExpandObjects", defaultValue="false") == "true"
-        )
         self._auto_expand_objects_action.setChecked(self.auto_expand_objects)
-        self._auto_expand_objects_action.toggled.connect(self.set_auto_expand_objects)
+        self._auto_expand_objects_action.triggered.connect(self._set_auto_expand_objects)
+        self._merge_dbs_action = self._menu.addAction("Merge databases")
+        self._merge_dbs_action.setCheckable(True)
+        self._merge_dbs_action.setChecked(self.merge_dbs)
+        self._merge_dbs_action.triggered.connect(self._set_merge_dbs)
+        self._max_rel_dim_action = ToolBarWidgetAction("Max relationship dimension", self._menu, compact=True)
+        self._max_rel_dim_spin_box = HorizontalSpinBox(self)
+        self._max_rel_dim_spin_box.setMinimum(2)
+        self._max_rel_dim_spin_box.setValue(self.max_relationship_dimension)
+        self._max_rel_dim_spin_box.valueChanged.connect(self._set_max_relationship_dimension)
+        self._max_rel_dim_action.tool_bar.addWidget(self._max_rel_dim_spin_box)
+        self._max_rel_dim_action.tool_bar.addSeparator()
+        self._disable_max_rel_dim_action = self._max_rel_dim_action.tool_bar.addAction("\u221E")
+        self._disable_max_rel_dim_action.setCheckable(True)
+        self._disable_max_rel_dim_action.toggled.connect(self._set_disable_max_relationship_dimension)
+        self._disable_max_rel_dim_action.toggled.connect(self._max_rel_dim_spin_box.setDisabled)
+        self._disable_max_rel_dim_action.setToolTip("No limit")
+        self._disable_max_rel_dim_action.setChecked(self.disable_max_relationship_dimension)
+        self._menu.addAction(self._max_rel_dim_action)
         self._menu.addSeparator()
         self._add_objects_action = self._menu.addAction("Add objects", self.add_objects_at_position)
         self._menu.addSeparator()
+        self._select_pos_param_action = self._menu.addAction(
+            "Select position parameters...", self.select_position_parameters
+        )
         self._save_pos_action = self._menu.addAction("Save positions", self.save_positions)
         self._clear_pos_action = self._menu.addAction("Clear saved positions", self.clear_saved_positions)
         self._menu.addSeparator()
@@ -185,24 +217,24 @@ class EntityQGraphicsView(CustomQGraphicsView):
         self._show_hidden_menu.setEnabled(any(self.hidden_items.values()))
         self._show_all_hidden_action.setEnabled(bool(self.hidden_items))
         self._prune_selected_action.setEnabled(bool(self.selected_items))
-        self._restore_pruned_menu.setEnabled(any(self.pruned_entity_ids.values()))
-        self._restore_all_pruned_action.setEnabled(any(self.pruned_entity_ids.values()))
+        self._restore_pruned_menu.setEnabled(any(self.pruned_db_map_entity_ids.values()))
+        self._restore_all_pruned_action.setEnabled(any(self.pruned_db_map_entity_ids.values()))
         self._prune_selected_action.setText(f"Prune {self._get_selected_entity_names()}")
         self._rebuild_action.setEnabled(has_graph)
         self._zoom_action.setEnabled(has_graph)
         self._rotate_action.setEnabled(has_graph)
         self._export_as_pdf_action.setEnabled(has_graph)
-        self._items_per_db_map_class = {}
+        self._items_per_class = {}
         for item in self.entity_items:
-            key = f"{item.entity_class_name}@{item.db_map.codename}"
-            self._items_per_db_map_class.setdefault(key, list()).append(item)
+            key = f"{item.entity_class_name}"
+            self._items_per_class.setdefault(key, list()).append(item)
         self._hide_classes_menu.clear()
-        self._hide_classes_menu.setEnabled(bool(self._items_per_db_map_class))
+        self._hide_classes_menu.setEnabled(bool(self._items_per_class))
         self._prune_classes_menu.clear()
-        self._prune_classes_menu.setEnabled(bool(self._items_per_db_map_class))
-        for key in sorted(self._items_per_db_map_class - self.hidden_items.keys()):
+        self._prune_classes_menu.setEnabled(bool(self._items_per_class))
+        for key in sorted(self._items_per_class.keys() - self.hidden_items.keys()):
             self._hide_classes_menu.addAction(key)
-        for key in sorted(self._items_per_db_map_class.keys() - self.pruned_entity_ids.keys()):
+        for key in sorted(self._items_per_class.keys() - self.pruned_db_map_entity_ids.keys()):
             self._prune_classes_menu.addAction(key)
         # FIXME: The heap map doesn't seem to be working nicely
         # self._parameter_heat_map_menu.setEnabled(has_graph)
@@ -223,11 +255,61 @@ class EntityQGraphicsView(CustomQGraphicsView):
         return menu
 
     @Slot(bool)
-    def set_auto_expand_objects(self, checked=False):
+    def _set_auto_expand_objects(self, _checked=False, save_setting=True):
+        checked = self._auto_expand_objects_action.isChecked()
+        if checked == self.auto_expand_objects:
+            return
+        if save_setting:
+            self._qsettings.setValue("appSettings/autoExpandObjects", "true" if checked else "false")
         self.auto_expand_objects = checked
-        self._auto_expand_objects_action.setChecked(checked)
         self._spine_db_editor.build_graph()
 
+    def set_auto_expand_objects(self, checked):
+        self._auto_expand_objects_action.setChecked(checked)
+        self._set_auto_expand_objects(save_setting=False)
+
+    @Slot(bool)
+    def _set_merge_dbs(self, _checked=False, save_setting=True):
+        checked = self._merge_dbs_action.isChecked()
+        if checked == self.merge_dbs:
+            return
+        if save_setting:
+            self._qsettings.setValue("appSettings/mergeDBs", "true" if checked else "false")
+        self.merge_dbs = checked
+        self._spine_db_editor.build_graph()
+
+    def set_merge_dbs(self, checked):
+        self._merge_dbs_action.setChecked(checked)
+        self._set_merge_dbs(save_setting=False)
+
+    @Slot(bool)
+    def _set_disable_max_relationship_dimension(self, _checked=False, save_setting=True):
+        checked = self._disable_max_rel_dim_action.isChecked()
+        if checked == self.disable_max_relationship_dimension:
+            return
+        if save_setting:
+            self._qsettings.setValue("appSettings/disableMaxRelationshipDimension", "true" if checked else "false")
+        self.disable_max_relationship_dimension = checked
+        self._spine_db_editor.build_graph()
+
+    def set_disable_max_relationship_dimension(self, checked):
+        self._disable_max_rel_dim_action.setChecked(checked)
+        self._set_disable_max_relationship_dimension(save_setting=False)
+
+    @Slot(int)
+    def _set_max_relationship_dimension(self, _value=None, save_setting=True):
+        value = self._max_rel_dim_spin_box.value()
+        if value == self.max_relationship_dimension:
+            return
+        if save_setting:
+            self._qsettings.setValue("appSettings/maxRelationshipDimension", str(value))
+        self.max_relationship_dimension = value
+        self._spine_db_editor.build_graph()
+
+    def set_max_relationship_dimension(self, value):
+        self._update_max_relationship_dimension_label(value)
+        self._set_max_relationship_dimension(save_setting=False)
+
     @Slot(bool)
     def add_objects_at_position(self, checked=False):
         self._spine_db_editor.add_objects_at_position(self._context_menu_pos)
@@ -245,11 +327,11 @@ class EntityQGraphicsView(CustomQGraphicsView):
         """Removes selected items."""
         if not self.selected_items:
             return
-        db_map_typed_data = {}
-        for item in self.selected_items:
-            db_map, entity_id = item.db_map_entity_id
-            db_map_typed_data.setdefault(db_map, {}).setdefault(item.entity_type, set()).add(entity_id)
-        self.db_mngr.remove_items(db_map_typed_data)
+        selected = {
+            "object": [item for item in self.selected_items if isinstance(item, ObjectItem)],
+            "relationship": [item for item in self.selected_items if isinstance(item, RelationshipItem)],
+        }
+        self._spine_db_editor.show_remove_entity_tree_items_form(selected)
 
     def _get_selected_entity_names(self):
         if not self.selected_items:
@@ -266,27 +348,28 @@ class EntityQGraphicsView(CustomQGraphicsView):
         self.hidden_items[key] = self.selected_items
         self._show_hidden_menu.addAction(key)
         for item in self.selected_items:
-            item.set_all_visible(False)
+            item.setVisible(False)
 
     @Slot("QAction")
     def _hide_class(self, action):
         """Hides some class."""
         key = action.text()
-        items = self._items_per_db_map_class[key]
+        items = self._items_per_class[key]
         self.hidden_items[key] = items
         self._show_hidden_menu.addAction(key)
         for item in items:
-            item.set_all_visible(False)
+            item.setVisible(False)
 
     @Slot(bool)
     def show_all_hidden_items(self, checked=False):
         """Shows all hidden items."""
         if not self.scene():
             return
+        self._show_hidden_menu.clear()
         while self.hidden_items:
             _, items = self.hidden_items.popitem()
             for item in items:
-                item.set_all_visible(True)
+                item.setVisible(True)
 
     @Slot("QAction")
     def show_hidden_items(self, action):
@@ -297,14 +380,14 @@ class EntityQGraphicsView(CustomQGraphicsView):
             action = next(iter(a for a in self._show_hidden_menu.actions() if a.text() == key))
             self._show_hidden_menu.removeAction(action)
             for item in items:
-                item.set_all_visible(True)
+                item.setVisible(True)
 
     @Slot(bool)
     def prune_selected_items(self, checked=False):
         """Prunes selected items."""
-        entity_ids = {x.db_map_entity_id for x in self.selected_items}
+        entity_ids = {db_map_id for x in self.selected_items for db_map_id in x.db_map_ids}
         key = self._get_selected_entity_names()
-        self.pruned_entity_ids[key] = entity_ids
+        self.pruned_db_map_entity_ids[key] = entity_ids
         self._restore_pruned_menu.addAction(key)
         self._spine_db_editor.build_graph()
 
@@ -312,13 +395,14 @@ class EntityQGraphicsView(CustomQGraphicsView):
     def _prune_class(self, action):
         """Prunnes some class."""
         key = action.text()
-        item = next(iter(self._items_per_db_map_class[key]))
-        class_id = item.entity_class_id
-        db_map = item.db_map
-        self.pruned_entity_ids[key] = {
+        self.pruned_db_map_entity_ids[key] = {
             (db_map, x["id"])
+            for item in self._items_per_class[key]
+            for db_map in item.db_maps
             for item_type in ("object", "relationship")
-            for x in self.db_mngr.get_items_by_field(db_map, item_type, "class_id", class_id)
+            for x in self.db_mngr.get_items_by_field(
+                db_map, item_type, "class_id", item.entity_class_id(db_map), only_visible=False
+            )
         }
         self._restore_pruned_menu.addAction(key)
         self._spine_db_editor.build_graph()
@@ -326,21 +410,22 @@ class EntityQGraphicsView(CustomQGraphicsView):
     @Slot(bool)
     def restore_all_pruned_items(self, checked=False):
         """Reinstates all pruned items."""
-        self.pruned_entity_ids.clear()
+        self.pruned_db_map_entity_ids.clear()
+        self._restore_pruned_menu.clear()
         self._spine_db_editor.build_graph()
 
     @Slot("QAction")
     def restore_pruned_items(self, action):
         """Reinstates some pruned items."""
         key = action.text()
-        if self.pruned_entity_ids.pop(key, None) is not None:
+        if self.pruned_db_map_entity_ids.pop(key, None) is not None:
             action = next(iter(a for a in self._restore_pruned_menu.actions() if a.text() == key))
             self._restore_pruned_menu.removeAction(action)
             self._spine_db_editor.build_graph()
 
     @Slot(bool)
     def select_position_parameters(self, checked=False):
-        dialog = SelectPositionParametersDialog(self._spine_db_editor)
+        dialog = SelectPositionParametersDialog(self._spine_db_editor, self.pos_x_parameter, self.pos_y_parameter)
         dialog.show()
         dialog.selection_made.connect(self._set_position_parameters)
 
@@ -360,31 +445,38 @@ class EntityQGraphicsView(CustomQGraphicsView):
         db_map_class_obj_items = {}
         db_map_class_rel_items = {}
         for item in obj_items:
-            db_map_class_obj_items.setdefault(item.db_map, {}).setdefault(item.entity_class_name, []).append(item)
+            for db_map in item.db_maps:
+                db_map_class_obj_items.setdefault(db_map, {}).setdefault(item.entity_class_name, []).append(item)
         for item in rel_items:
-            db_map_class_rel_items.setdefault(item.db_map, {}).setdefault(item.entity_class_name, []).append(item)
+            for db_map in item.db_maps:
+                db_map_class_rel_items.setdefault(db_map, {}).setdefault(item.entity_class_name, []).append(item)
         db_map_data = {}
         for db_map, class_obj_items in db_map_class_obj_items.items():
             data = db_map_data.setdefault(db_map, {})
             for class_name, obj_items in class_obj_items.items():
-                data["object_parameters"] = [(class_name, self.pos_x_parameter), (class_name, self.pos_y_parameter)]
-                data["object_parameter_values"] = [
-                    (class_name, item.entity_name, self.pos_x_parameter, item.pos().x()) for item in obj_items
-                ] + [(class_name, item.entity_name, self.pos_y_parameter, item.pos().y()) for item in obj_items]
+                data.setdefault("object_parameters", []).extend(
+                    [(class_name, self.pos_x_parameter), (class_name, self.pos_y_parameter)]
+                )
+                data.setdefault("object_parameter_values", []).extend(
+                    [(class_name, item.entity_name, self.pos_x_parameter, item.pos().x()) for item in obj_items]
+                    + [(class_name, item.entity_name, self.pos_y_parameter, item.pos().y()) for item in obj_items]
+                )
         for db_map, class_rel_items in db_map_class_rel_items.items():
             data = db_map_data.setdefault(db_map, {})
             for class_name, rel_items in class_rel_items.items():
-                data["relationship_parameters"] = [
-                    (class_name, self.pos_x_parameter),
-                    (class_name, self.pos_y_parameter),
-                ]
-                data["relationship_parameter_values"] = [
-                    (class_name, item.object_name_list.split(","), self.pos_x_parameter, item.pos().x())
-                    for item in rel_items
-                ] + [
-                    (class_name, item.object_name_list.split(","), self.pos_y_parameter, item.pos().y())
-                    for item in rel_items
-                ]
+                data.setdefault("relationship_parameters", []).extend(
+                    [(class_name, self.pos_x_parameter), (class_name, self.pos_y_parameter)]
+                )
+                data.setdefault("relationship_parameter_values", []).extend(
+                    [
+                        (class_name, item.object_name_list.split(","), self.pos_x_parameter, item.pos().x())
+                        for item in rel_items
+                    ]
+                    + [
+                        (class_name, item.object_name_list.split(","), self.pos_y_parameter, item.pos().y())
+                        for item in rel_items
+                    ]
+                )
         self.db_mngr.import_data(db_map_data)
 
     @Slot(bool)
@@ -395,14 +487,13 @@ class EntityQGraphicsView(CustomQGraphicsView):
         for item in self.selected_items:
             db_map_ids.setdefault(item.db_map, set()).add(item.entity_id)
         db_map_typed_data = {}
-        # FIXME: We might need to fetch all parameter values here!!!
         for db_map, ids in db_map_ids.items():
             db_map_typed_data[db_map] = {
                 "parameter_value": set(
                     pv["id"]
                     for parameter_name in (self.pos_x_parameter, self.pos_y_parameter)
                     for pv in self.db_mngr.get_items_by_field(
-                        db_map, "parameter_value", "parameter_name", parameter_name
+                        db_map, "parameter_value", "parameter_name", parameter_name, only_visible=False
                     )
                     if pv["entity_id"] in ids
                 )
@@ -465,8 +556,7 @@ class EntityQGraphicsView(CustomQGraphicsView):
 
     @Slot("QAction")
     def add_heat_map(self, action):
-        """Adds heat map for the parameter in the action text.
-        """
+        """Adds heat map for the parameter in the action text."""
         self._clean_up_heat_map_items()
         point_value_tuples = self._point_value_tuples_per_parameter_name[action.text()]
         x, y, values = zip(*point_value_tuples)
@@ -514,13 +604,12 @@ class EntityQGraphicsView(CustomQGraphicsView):
         self.viewport().unsetCursor()
 
     def _cross_hairs_has_valid_target(self):
-        return (
-            self._hovered_obj_item.db_map == self.cross_hairs_items[0].db_map
-            and self._hovered_obj_item.entity_class_id in self.relationship_class["object_class_ids_to_go"]
-        )
+        db_map = self.relationship_class["db_map"]
+        return self._hovered_obj_item.entity_class_id(db_map) in self.relationship_class["object_class_ids_to_go"]
 
     def mousePressEvent(self, event):
         """Handles relationship creation if one it's in process."""
+        self._previous_mouse_pos = event.pos()
         if not self.cross_hairs_items:
             super().mousePressEvent(event)
             return
@@ -528,7 +617,8 @@ class EntityQGraphicsView(CustomQGraphicsView):
             self.clear_cross_hairs_items()
             return
         if self._cross_hairs_has_valid_target():
-            self.relationship_class["object_class_ids_to_go"].remove(self._hovered_obj_item.entity_class_id)
+            db_map = self.relationship_class["db_map"]
+            self.relationship_class["object_class_ids_to_go"].remove(self._hovered_obj_item.entity_class_id(db_map))
             if self.relationship_class["object_class_ids_to_go"]:
                 # Add hovered as member and keep going, we're not done yet
                 ch_rel_item = self.cross_hairs_items[1]
@@ -551,7 +641,11 @@ class EntityQGraphicsView(CustomQGraphicsView):
             self._update_cross_hairs_pos(event.pos())
             return
         super().mouseMoveEvent(event)
-        if not self.itemAt(event.pos()) and (event.buttons() & Qt.LeftButton != 0):
+        if (
+            not self.itemAt(event.pos())
+            and (event.buttons() & Qt.LeftButton != 0)
+            and self.dragMode() != self.RubberBandDrag
+        ):
             if self._previous_mouse_pos is not None:
                 delta = event.pos() - self._previous_mouse_pos
                 self._scroll_scene_by(delta.x(), delta.y())
@@ -582,6 +676,7 @@ class EntityQGraphicsView(CustomQGraphicsView):
         self.cross_hairs_items[0].set_normal_icon()
 
     def mouseReleaseEvent(self, event):
+        self._previous_mouse_pos = None
         if not self.cross_hairs_items:
             super().mouseReleaseEvent(event)
 
