@@ -11,14 +11,13 @@
 
 import os
 import uuid
-from concurrent.futures import ThreadPoolExecutor
 from pygments.styles import get_style_by_name
 from pygments.lexers import get_lexer_by_name
 from pygments.util import ClassNotFound
 from pygments.token import Token
-from PySide2.QtCore import Qt, Slot, QTimer, Signal, QThreadPool, QRunnable
+from PySide2.QtCore import Qt, Slot, QTimer, Signal, QThreadPool, QRunnable, QMutex, QMutexLocker
 from PySide2.QtWidgets import QTextEdit
-from PySide2.QtGui import QFontDatabase, QTextCharFormat, QFont, QTextCursor, QColor, QTextBlockFormat
+from PySide2.QtGui import QFontDatabase, QTextCharFormat, QFont, QTextCursor, QColor, QTextBlockFormat, QTextOption
 from spinetoolbox.helpers import CustomSyntaxHighlighter
 from spinetoolbox.spine_engine_manager import make_engine_manager
 
@@ -26,7 +25,7 @@ from spinetoolbox.spine_engine_manager import make_engine_manager
 class PersistentConsoleWidget(QTextEdit):
     """A widget to interact with a persistent process."""
 
-    _history_item_available = Signal(str)
+    _history_item_available = Signal(str, str)
     _completions_available = Signal(str, str, list)
     _FLUSH_INTERVAL = 200
     _MAX_LINES_PER_SECOND = 2000
@@ -46,19 +45,22 @@ class PersistentConsoleWidget(QTextEdit):
         cursor_width = self.fontMetrics().horizontalAdvance("x")
         self.setCursorWidth(cursor_width)
         self.document().setIndentWidth(cursor_width)
+        self.document().setMaximumBlockCount(2000)
+        self.setWordWrapMode(QTextOption.WrapAnywhere)
         self.setTabStopDistance(4 * cursor_width)
         self._pool = QThreadPool()
-        self._pool.setMaxThreadCount(1)
-        self._executor = ThreadPoolExecutor()
+        self._text_buffer_pool = QThreadPool()
+        self._text_buffer_pool.setMaxThreadCount(1)
         self._toolbox = toolbox
         self._key = key
         self._language = language
         self.owners = {owner}
         self._prompt, self._prompt_format = self._make_prompt()
-        self._history_index = 0
-        self._history_item_zero = ""
+        self._prefix = None
+        self._reset_prefix = True
         self._pending_command_count = 0
         self._text_buffer = []
+        self._text_buffer_mutex = QMutex()
         self._skipped = {}
         self._anchor = None
         self._flush_timer = QTimer()
@@ -140,6 +142,8 @@ class PersistentConsoleWidget(QTextEdit):
     @Slot()
     def _handle_cursor_position_changed(self):
         self.setReadOnly(self.textCursor().position() < self._input_start_pos)
+        if self._reset_prefix:
+            self._prefix = None
 
     def _make_prompt(self):
         text_format = QTextCharFormat()
@@ -211,14 +215,19 @@ class PersistentConsoleWidget(QTextEdit):
         Args:
             text (str)
         """
+        self._text_buffer_pool.start(_CustomRunnable(self._do_insert_text_before_prompt, text, with_prompt))
+
+    def _do_insert_text_before_prompt(self, text, with_prompt=False):
+        QMutexLocker(self._text_buffer_mutex)
         self._text_buffer.append((text, with_prompt))
 
     @Slot()
     def _flush_text_buffer(self):
         """Inserts all text from buffer."""
-        self._pool.start(_CustomRunnable(self._do_flush_text_buffer))
+        self._text_buffer_pool.start(_CustomRunnable(self._do_flush_text_buffer))
 
     def _do_flush_text_buffer(self):
+        QMutexLocker(self._text_buffer_mutex)
         self.blockSignals(True)
         k = 0
         cursor = self.textCursor()
@@ -281,17 +290,20 @@ class PersistentConsoleWidget(QTextEdit):
 
         Returns:
             str: the complete text
-            str: the text before the cursor (for autocompletion)
         """
         cursor = self.textCursor()
-        cursor.movePosition(QTextCursor.StartOfBlock, QTextCursor.KeepAnchor)
-        partial_text = cursor.selectedText()
         cursor.setPosition(self._input_start_pos)
         cursor.movePosition(QTextCursor.End, QTextCursor.KeepAnchor)
-        text = cursor.selectedText()
-        return text, partial_text
+        return cursor.selectedText()
 
-    def _highlight_current_text(self):
+    def _get_prefix(self):
+        if self._prefix is None:
+            cursor = self.textCursor()
+            cursor.setPosition(self._input_start_pos, QTextCursor.KeepAnchor)
+            self._prefix = cursor.selectedText()
+        return self._prefix
+
+    def _highlight_current_input(self):
         cursor = self.textCursor()
         cursor.setPosition(self._input_start_pos)
         cursor.movePosition(QTextCursor.End, QTextCursor.KeepAnchor)
@@ -306,22 +318,22 @@ class PersistentConsoleWidget(QTextEdit):
 
     def keyPressEvent(self, ev):
         self._at_bottom = True
-        text, partial_text = self._get_current_text()
+        text = self._get_current_text()
         if ev.key() in (Qt.Key_Return, Qt.Key_Enter):
             self._issue_command(text)
         elif ev.key() == Qt.Key_Up:
-            self._move_history(text, 1)
+            self._move_history(text, True)
         elif ev.key() == Qt.Key_Down:
-            self._move_history(text, -1)
-        elif ev.key() == Qt.Key_Tab and partial_text.strip():
-            self._autocomplete(text, partial_text)
+            self._move_history(text, False)
+        elif ev.key() == Qt.Key_Tab:
+            self._autocomplete(text)
         elif ev.key() != Qt.Key_Backspace or self.textCursor().position() > self._input_start_pos:
-            super().keyPressEvent(ev)
             cursor = self.textCursor()
-            if cursor.position() < self._input_start_pos:
+            super().keyPressEvent(ev)
+            if cursor.position() >= self._input_start_pos and self.textCursor().position() < self._input_start_pos:
                 cursor.setPosition(self._input_start_pos)
                 self.setTextCursor(cursor)
-        self._highlight_current_text()
+            self._highlight_current_input()
 
     def _issue_command(self, text):
         """Issues command.
@@ -336,9 +348,8 @@ class PersistentConsoleWidget(QTextEdit):
             return
         log_stdin = bool(self._pending_command_count)
         self._make_prompt_block(prompt="")
-        self._history_index = 0
         self._pending_command_count += 1
-        self._executor.submit(self._do_issue_command, engine_mngr, text, log_stdin)
+        self._pool.start(_CustomRunnable(self._do_issue_command, engine_mngr, text, log_stdin))
 
     def _do_issue_command(self, engine_mngr, text, log_stdin):
         for msg in engine_mngr.issue_persistent_command(self._key, text):
@@ -356,63 +367,59 @@ class PersistentConsoleWidget(QTextEdit):
         if self._pending_command_count == 0:
             self._insert_prompt(prompt=self._prompt)
 
-    def _move_history(self, text, step):
-        """Moves history.
+    def _move_history(self, text, backwards):
+        """Moves history."""
+        self._pool.start(_CustomRunnable(self._do_move_history, text, backwards))
 
-        Args:
-            text (str)
-            step (int)
-        """
-        if self._history_index == 0:
-            self._history_item_zero = text
-        self._history_index += step
-        if self._history_index < 1:
-            self._history_index = 0
-            history_item = self._history_item_zero
-            self._display_history_item(history_item)
-            return
-        self._executor.submit(self._do_move_history)
-
-    def _do_move_history(self):
+    def _do_move_history(self, text, backwards):
         engine_server_address = self._toolbox.qsettings().value("appSettings/engineServerAddress", defaultValue="")
         engine_mngr = make_engine_manager(engine_server_address)
-        history_item = engine_mngr.get_persistent_history_item(self._key, self._history_index)
-        self._history_item_available.emit(history_item)
+        prefix = self._get_prefix()
+        history_item = engine_mngr.get_persistent_history_item(self._key, text, prefix, backwards)
+        self._history_item_available.emit(history_item, prefix)
 
-    @Slot(str)
-    def _display_history_item(self, history_item):
+    @Slot(str, str)
+    def _display_history_item(self, history_item, prefix):
+        self._reset_prefix = False
         cursor = self.textCursor()
-        cursor.setPosition(self._input_start_pos)
+        pos = self._input_start_pos + len(prefix)
+        cursor.setPosition(pos)
         cursor.movePosition(QTextCursor.End, QTextCursor.KeepAnchor)
-        self._insert_stdin_text(cursor, history_item)
+        self._insert_stdin_text(cursor, history_item[len(prefix) :])
+        if prefix:
+            cursor.setPosition(pos)
+        self.setTextCursor(cursor)
+        self._reset_prefix = True
+        self._highlight_current_input()
 
-    def _autocomplete(self, text, partial_text):
+    def _autocomplete(self, text):
         """Autocompletes current text in the prompt (or output options if multiple matches).
 
         Args:
             text (str)
-            partial_text (str)
         """
-        self._executor.submit(self._do_autocomplete, text, partial_text)
+        self._pool.start(_CustomRunnable(self._do_autocomplete, text))
 
-    def _do_autocomplete(self, text, partial_text):
+    def _do_autocomplete(self, text):
         engine_server_address = self._toolbox.qsettings().value("appSettings/engineServerAddress", defaultValue="")
         engine_mngr = make_engine_manager(engine_server_address)
-        completions = engine_mngr.get_persistent_completions(self._key, partial_text)
-        self._completions_available.emit(text, partial_text, completions)
+        prefix = self._get_prefix()
+        completions = engine_mngr.get_persistent_completions(self._key, prefix)
+        self._completions_available.emit(text, prefix, completions)
 
     @Slot(str, str, list)
-    def _display_completions(self, text, partial_text, completions):
-        prefix = os.path.commonprefix(completions)
-        if partial_text.endswith(prefix) and len(completions) > 1:
+    def _display_completions(self, text, prefix, completions):
+        completion = os.path.commonprefix(completions)
+        if prefix.endswith(completion) and len(completions) > 1:
             # Can't complete, but there is more than one option: 'commit' stdin and output options to stdout
             self.add_stdin(text)
             self.add_stdout("\t\t".join(completions))
         else:
             # Complete in current line
             cursor = self.textCursor()
-            last_word = partial_text.split(" ")[-1]
-            cursor.insertText(prefix[len(last_word) :])
+            last_prefix_word = prefix.split(" ")[-1]
+            cursor.insertText(completion[len(last_prefix_word) :])
+            self._highlight_current_input()
 
     @Slot(bool)
     def _restart_persistent(self, _=False):
@@ -420,7 +427,7 @@ class PersistentConsoleWidget(QTextEdit):
         self.clear()
         engine_server_address = self._toolbox.qsettings().value("appSettings/engineServerAddress", defaultValue="")
         engine_mngr = make_engine_manager(engine_server_address)
-        self._executor.submit(self._do_restart_persistent, engine_mngr)
+        self._pool.start(_CustomRunnable(self._do_restart_persistent, engine_mngr))
 
     def _do_restart_persistent(self, engine_mngr):
         for msg in engine_mngr.restart_persistent(self._key):
@@ -436,7 +443,7 @@ class PersistentConsoleWidget(QTextEdit):
         """Interrupts underlying persistent process."""
         engine_server_address = self._toolbox.qsettings().value("appSettings/engineServerAddress", defaultValue="")
         engine_mngr = make_engine_manager(engine_server_address)
-        self._executor.submit(engine_mngr.interrupt_persistent, self._key)
+        self._pool.start(_CustomRunnable(engine_mngr.interrupt_persistent, self._key))
 
     def _extend_menu(self, menu):
         """Adds two more actions: Restart, and Interrupt."""
@@ -452,12 +459,14 @@ class PersistentConsoleWidget(QTextEdit):
 
 
 class _CustomRunnable(QRunnable):
-    def __init__(self, function_to_run):
+    def __init__(self, function_to_run, *args, **kwargs):
         super().__init__(self)
         self._function_to_run = function_to_run
+        self._args = args
+        self._kwargs = kwargs
 
     def run(self):
-        self._function_to_run()
+        self._function_to_run(*self._args, **self._kwargs)
 
 
 # Translated from
