@@ -16,10 +16,10 @@ Functions to make and handle QToolBars.
 :date:   19.1.2018
 """
 
-from PySide2.QtCore import Qt
-from PySide2.QtWidgets import QToolBar, QLabel, QToolButton, QAbstractButton
+from PySide2.QtCore import Qt, Slot
+from PySide2.QtWidgets import QToolBar, QLabel, QToolButton
 from PySide2.QtGui import QIcon, QPainter
-from ..helpers import make_icon_toolbar_ss, ColoredIcon
+from ..helpers import make_icon_toolbar_ss, ColoredIcon, CharIconEngine
 from .project_item_drag import ProjectItemButton, ProjectItemSpecButton, ProjectItemSpecArray
 
 
@@ -50,8 +50,9 @@ class ToolBar(QToolBar):
         Args:
             enabled (bool): True to enable actions, False to disable
         """
-        for button in self.findChildren(QAbstractButton):
-            button.setEnabled(enabled)
+        for child_type in (ProjectItemButton, ProjectItemSpecButton):
+            for button in self.findChildren(child_type):
+                button.setEnabled(enabled)
 
 
 class PluginToolBar(ToolBar):
@@ -65,6 +66,8 @@ class PluginToolBar(ToolBar):
         """
         super().__init__(name, parent)  # Inherits stylesheet from ToolboxUI
         self._name = name
+        self._buttons = {}
+        self._toolbox.specification_model.specification_replaced.connect(self._update_spec_button_name)
 
     def setup(self, plugin_specs, disabled_names):
         """Sets up the toolbar.
@@ -83,10 +86,19 @@ class PluginToolBar(ToolBar):
                 if spec.name in disabled_names:
                     button.setEnabled(False)
                 self.addWidget(button)
+                self._buttons[spec.name] = button
 
     def set_color(self, color):
         self.setStyleSheet(make_icon_toolbar_ss(color))
 
+    @Slot(str, str)
+    def _update_spec_button_name(self, old_name, new_name):
+        button = self._buttons.pop(old_name, None)
+        if button is None:
+            return
+        self._buttons[new_name] = button
+        button.spec_name = new_name
+
 
 class MainToolBar(ToolBar):
     """The main application toolbar: Items | Execute"""
@@ -114,6 +126,11 @@ class MainToolBar(ToolBar):
         self._drop_target_action = None
         self.setAcceptDrops(True)
 
+    def set_project_actions_enabled(self, enabled):
+        super().set_project_actions_enabled(enabled)
+        for arr in self._spec_arrays:
+            arr.update()
+
     def set_color(self, color):
         self.setStyleSheet(make_icon_toolbar_ss(color))
         self.layout().setSpacing(1)
@@ -130,16 +147,22 @@ class MainToolBar(ToolBar):
         icon_ordering = self._toolbox.qsettings().value("appSettings/toolbarIconOrdering", defaultValue="")
         ordered_item_types = icon_ordering.split(self._SEPARATOR)
         for item_type in ordered_item_types:
+            if item_type == "Gimlet":  # Do not allow creating new Gimlets
+                continue
             factory = self._toolbox.item_factories.get(item_type)
             if factory is None:
                 continue
             self._add_project_item_button(item_type, factory, colored)
         for item_type, factory in self._toolbox.item_factories.items():
+            if item_type == "Gimlet":
+                continue
             if item_type in ordered_item_types:
                 continue
             self._add_project_item_button(item_type, factory, colored)
         self._make_tool_button(
-            QIcon(":/icons/wrench_plus.svg"), "Add specification from file...", self._toolbox.import_specification
+            QIcon(CharIconEngine("\uf067", color=Qt.darkGreen)),
+            "Add specification from file...",
+            self._toolbox.import_specification,
         )
 
     def _add_project_item_button(self, item_type, factory, colored):
