@@ -21,7 +21,7 @@ from unittest import mock
 import logging
 import sys
 from PySide2.QtWidgets import QApplication
-from PySide2.QtCore import QItemSelectionModel
+from PySide2.QtCore import QModelIndex, QItemSelectionModel
 import spinetoolbox.resources_icons_rc  # pylint: disable=unused-import
 from spinetoolbox.spine_db_manager import SpineDBManager
 from spinetoolbox.spine_db_editor.widgets.spine_db_editor import SpineDBEditor
@@ -45,17 +45,26 @@ class TestSpineDBEditor(
 
     @staticmethod
     def _object(*args):
-        return dict(zip(["id", "class_id", "name", "description"], args))
+        return dict(zip(["id", "class_id", "class_name", "name", "description"], args))
 
     @staticmethod
     def _relationship_class(*args):
-        return dict(zip(["id", "name", "object_class_id_list", "object_class_name_list"], args))
+        return dict(zip(["id", "name", "object_class_id_list", "object_class_name_list", "display_icon"], args))
 
     @staticmethod
     def _relationship(*args):
         return dict(
             zip(
-                ["id", "class_id", "name", "class_name", "object_class_id_list", "object_id_list", "object_name_list"],
+                [
+                    "id",
+                    "class_id",
+                    "name",
+                    "class_name",
+                    "object_class_id_list",
+                    "object_class_name_list",
+                    "object_id_list",
+                    "object_name_list",
+                ],
                 args,
             )
         )
@@ -63,6 +72,7 @@ class TestSpineDBEditor(
     @staticmethod
     def _object_parameter_definition(*args):
         d = dict(zip(["id", "object_class_id", "object_class_name", "parameter_name"], args))
+        d.update({"default_value": None, "default_type": None})
         return d
 
     @staticmethod
@@ -80,6 +90,7 @@ class TestSpineDBEditor(
                 args,
             )
         )
+        d.update({"default_value": None, "default_type": None})
         return d
 
     @staticmethod
@@ -94,6 +105,7 @@ class TestSpineDBEditor(
                     "object_name",
                     "parameter_id",
                     "parameter_name",
+                    "alternative_name",
                     "value",
                     "type",
                 ],
@@ -118,6 +130,7 @@ class TestSpineDBEditor(
                     "object_name_list",
                     "parameter_id",
                     "parameter_name",
+                    "alternative_name",
                     "value",
                     "type",
                 ],
@@ -151,22 +164,25 @@ class TestSpineDBEditor(
             "fish__dog",
             str(cls.fish_class["id"]) + "," + str(cls.dog_class["id"]),
             cls.fish_class["name"] + "," + cls.dog_class["name"],
+            None,
         )
         cls.dog_fish_class = cls._relationship_class(
             4,
             "dog__fish",
             str(cls.dog_class["id"]) + "," + str(cls.fish_class["id"]),
             cls.dog_class["name"] + "," + cls.fish_class["name"],
+            None,
         )
-        cls.nemo_object = cls._object(1, cls.fish_class["id"], 'nemo', 'The lost one.')
-        cls.pluto_object = cls._object(2, cls.dog_class["id"], 'pluto', "Mickey's.")
-        cls.scooby_object = cls._object(3, cls.dog_class["id"], 'scooby', 'Scooby-Dooby-Doo.')
+        cls.nemo_object = cls._object(1, cls.fish_class["id"], cls.fish_class["name"], 'nemo', 'The lost one.')
+        cls.pluto_object = cls._object(2, cls.dog_class["id"], cls.dog_class["name"], 'pluto', "Mickey's.")
+        cls.scooby_object = cls._object(3, cls.dog_class["id"], cls.dog_class["name"], 'scooby', 'Scooby-Dooby-Doo.')
         cls.pluto_nemo_rel = cls._relationship(
             4,
             cls.dog_fish_class["id"],
             "dog__fish_pluto__nemo",
             cls.dog_fish_class["name"],
             str(cls.dog_class["id"]) + "," + str(cls.fish_class["id"]),
+            cls.dog_class["name"] + "," + cls.fish_class["name"],
             str(cls.pluto_object["id"]) + "," + str(cls.nemo_object["id"]),
             cls.pluto_object["name"] + "," + cls.nemo_object["name"],
         )
@@ -176,6 +192,7 @@ class TestSpineDBEditor(
             "fish__dog_nemo__pluto",
             cls.fish_dog_class["name"],
             str(cls.fish_class["id"]) + "," + str(cls.dog_class["id"]),
+            cls.fish_class["name"] + "," + cls.dog_class["name"],
             str(cls.nemo_object["id"]) + "," + str(cls.pluto_object["id"]),
             cls.nemo_object["name"] + "," + cls.pluto_object["name"],
         )
@@ -185,6 +202,7 @@ class TestSpineDBEditor(
             "fish__dog_nemo__scooby",
             cls.fish_dog_class["name"],
             str(cls.fish_class["id"]) + "," + str(cls.dog_class["id"]),
+            cls.fish_class["name"] + "," + cls.dog_class["name"],
             str(cls.nemo_object["id"]) + "," + str(cls.scooby_object["id"]),
             cls.nemo_object["name"] + "," + cls.scooby_object["name"],
         )
@@ -214,6 +232,7 @@ class TestSpineDBEditor(
             cls.nemo_object["name"],
             cls.water_parameter["id"],
             cls.water_parameter["parameter_name"],
+            "Base",
             b'"salt"',
             None,
         )
@@ -225,6 +244,7 @@ class TestSpineDBEditor(
             cls.pluto_object["name"],
             cls.breed_parameter["id"],
             cls.breed_parameter["parameter_name"],
+            "Base",
             b'"bloodhound"',
             None,
         )
@@ -236,6 +256,7 @@ class TestSpineDBEditor(
             cls.scooby_object["name"],
             cls.breed_parameter["id"],
             cls.breed_parameter["parameter_name"],
+            "Base",
             b'"great dane"',
             None,
         )
@@ -250,6 +271,7 @@ class TestSpineDBEditor(
             cls.nemo_pluto_rel["object_name_list"],
             cls.relative_speed_parameter["id"],
             cls.relative_speed_parameter["parameter_name"],
+            "Base",
             b"-1",
             None,
         )
@@ -264,6 +286,7 @@ class TestSpineDBEditor(
             cls.nemo_scooby_rel["object_name_list"],
             cls.relative_speed_parameter["id"],
             cls.relative_speed_parameter["parameter_name"],
+            "Base",
             b"5",
             None,
         )
@@ -278,6 +301,7 @@ class TestSpineDBEditor(
             cls.pluto_nemo_rel["object_name_list"],
             cls.combined_mojo_parameter["id"],
             cls.combined_mojo_parameter["parameter_name"],
+            "Base",
             b"100",
             None,
         )
@@ -312,10 +336,7 @@ class TestSpineDBEditor(
     def put_mock_object_classes_in_db_mngr(self):
         """Put fish and dog object classes in the db mngr."""
         object_classes = [self.fish_class, self.dog_class]
-        with mock.patch("spinetoolbox.spine_db_manager.SpineDBManager.entity_class_icon") as mock_icon:
-            mock_icon.return_value = None
-            self.db_mngr.object_classes_added.emit({self.mock_db_map: object_classes})
-            mock_icon.assert_called()
+        self.db_mngr.object_classes_added.emit({self.mock_db_map: object_classes})
 
     def put_mock_objects_in_db_mngr(self):
         """Put nemo, pluto and scooby objects in the db mngr."""
@@ -325,10 +346,7 @@ class TestSpineDBEditor(
     def put_mock_relationship_classes_in_db_mngr(self):
         """Put dog__fish and fish__dog relationship classes in the db mngr."""
         relationship_classes = [self.fish_dog_class, self.dog_fish_class]
-        with mock.patch("spinetoolbox.spine_db_manager.SpineDBManager.entity_class_icon") as mock_icon:
-            mock_icon.return_value = None
-            self.db_mngr.relationship_classes_added.emit({self.mock_db_map: relationship_classes})
-            mock_icon.assert_called()
+        self.db_mngr.relationship_classes_added.emit({self.mock_db_map: relationship_classes})
 
     def put_mock_relationships_in_db_mngr(self):
         """Put pluto_nemo, nemo_pluto and nemo_scooby relationships in the db mngr."""
@@ -393,7 +411,7 @@ class TestSpineDBEditor(
         self.spine_db_editor.ui.treeView_object.selectionModel().select(fish_index, QItemSelectionModel.Select)
         # Check default in object parameter_definition
         model = self.spine_db_editor.object_parameter_definition_model
-        model.empty_model.fetchMore()
+        model.empty_model.fetchMore(QModelIndex())
         h = model.header.index
         row_data = []
         for row in range(model.rowCount()):
